@@ -2,10 +2,35 @@ package Test2::API;
 use strict;
 use warnings;
 
+our $VERSION = '0.000044';
+
 my $INST;
+my $ENDING = 0;
+sub test2_set_is_end { ($ENDING) = @_ ? @_ : (1) }
+sub test2_get_is_end { $ENDING }
+
 use Test2::API::Instance(\$INST);
 # Set the exit status
-END { $INST->set_exit() }
+END {
+    test2_set_is_end(); # See gh #16
+    $INST->set_exit();
+}
+
+# See gh #16
+{
+    no warnings;
+    INIT { eval 'END { test2_set_is_end() }; 1' or die $@ }
+}
+
+BEGIN {
+    no warnings 'once';
+    if($] ge '5.014' || $ENV{T2_CHECK_DEPTH} || $Test2::API::DO_DEPTH_CHECK) {
+        *DO_DEPTH_CHECK = sub() { 1 };
+    }
+    else {
+        *DO_DEPTH_CHECK = sub() { 0 };
+    }
+}
 
 use Test2::Util::Trace();
 
@@ -37,17 +62,22 @@ our @EXPORT_OK = qw{
     test2_init_done
     test2_load_done
 
+    test2_set_is_end
+    test2_get_is_end
+
     test2_pid
     test2_tid
     test2_stack
     test2_no_wait
 
     test2_add_callback_context_aquire
+    test2_add_callback_context_acquire
     test2_add_callback_context_init
     test2_add_callback_context_release
     test2_add_callback_exit
     test2_add_callback_post_load
     test2_list_context_aquire_callbacks
+    test2_list_context_acquire_callbacks
     test2_list_context_init_callbacks
     test2_list_context_release_callbacks
     test2_list_exit_callbacks
@@ -89,10 +119,10 @@ sub import {
     goto &import;
 }
 
-my $STACK      = $INST->stack;
-my $CONTEXTS   = $INST->contexts;
-my $INIT_CBS   = $INST->context_init_callbacks;
-my $AQUIRE_CBS = $INST->context_aquire_callbacks;
+my $STACK       = $INST->stack;
+my $CONTEXTS    = $INST->contexts;
+my $INIT_CBS    = $INST->context_init_callbacks;
+my $ACQUIRE_CBS = $INST->context_acquire_callbacks;
 
 sub test2_init_done { $INST->finalized }
 sub test2_load_done { $INST->loaded }
@@ -105,12 +135,14 @@ sub test2_no_wait {
     $INST->no_wait;
 }
 
-sub test2_add_callback_context_aquire    { $INST->add_context_aquire_callback(@_) }
+sub test2_add_callback_context_acquire   { $INST->add_context_acquire_callback(@_) }
+sub test2_add_callback_context_aquire    { $INST->add_context_acquire_callback(@_) }
 sub test2_add_callback_context_init      { $INST->add_context_init_callback(@_) }
 sub test2_add_callback_context_release   { $INST->add_context_release_callback(@_) }
 sub test2_add_callback_exit              { $INST->add_exit_callback(@_) }
 sub test2_add_callback_post_load         { $INST->add_post_load_callback(@_) }
-sub test2_list_context_aquire_callbacks  { @{$INST->context_aquire_callbacks} }
+sub test2_list_context_aquire_callbacks  { @{$INST->context_acquire_callbacks} }
+sub test2_list_context_acquire_callbacks { @{$INST->context_acquire_callbacks} }
 sub test2_list_context_init_callbacks    { @{$INST->context_init_callbacks} }
 sub test2_list_context_release_callbacks { @{$INST->context_release_callbacks} }
 sub test2_list_exit_callbacks            { @{$INST->exit_callbacks} }
@@ -130,15 +162,15 @@ sub test2_formatter     { $INST->formatter }
 sub test2_formatters    { @{$INST->formatters} }
 sub test2_formatter_add { $INST->add_formatter(@_) }
 sub test2_formatter_set {
-    my ($formater) = @_;
-    croak "No formatter specified" unless $formater;
+    my ($formatter) = @_;
+    croak "No formatter specified" unless $formatter;
     croak "Global Formatter already set" if $INST->formatter_set;
-    $INST->set_formatter($formater);
+    $INST->set_formatter($formatter);
 }
 
 # Private, for use in Test2::API::Context
 sub _contexts_ref                  { $INST->contexts }
-sub _context_aquire_callbacks_ref  { $INST->context_aquire_callbacks }
+sub _context_acquire_callbacks_ref { $INST->context_acquire_callbacks }
 sub _context_init_callbacks_ref    { $INST->context_init_callbacks }
 sub _context_release_callbacks_ref { $INST->context_release_callbacks }
 
@@ -207,21 +239,27 @@ sub context {
     my $hid     = $hub->{hid};
     my $current = $CONTEXTS->{$hid};
 
-    $_->(\%params) for @$AQUIRE_CBS;
-    map $_->(\%params), @{$hub->{_context_aquire}} if $hub->{_context_aquire};
+    $_->(\%params) for @$ACQUIRE_CBS;
+    map $_->(\%params), @{$hub->{_context_acquire}} if $hub->{_context_acquire};
+
+    # This is for https://github.com/Test-More/test-more/issues/16
+    # and https://rt.perl.org/Public/Bug/Display.html?id=127774
+    my $phase = ${^GLOBAL_PHASE} || 'NA';
+    my $end_phase = $ENDING || $phase eq 'END' || $phase eq 'DESTRUCT';
 
     my $level = 1 + $params{level};
-    my ($pkg, $file, $line, $sub) = caller($level);
-    unless ($pkg) {
+    my ($pkg, $file, $line, $sub) = $end_phase ? caller(0) : caller($level);
+    unless ($pkg || $end_phase) {
         confess "Could not find context at depth $level" unless $params{fudge};
         ($pkg, $file, $line, $sub) = caller(--$level) while ($level >= 0 && !$pkg);
     }
 
     my $depth = $level;
-    $depth++ while caller($depth + 1) && (!$current || $depth <= $current->{_depth} + $params{wrapped});
+    $depth++ while DO_DEPTH_CHECK && !$end_phase && (!$current || $depth <= $current->{_depth} + $params{wrapped}) && caller($depth + 1);
     $depth -= $params{wrapped};
+    my $depth_ok = !DO_DEPTH_CHECK || $end_phase || !$current || $current->{_depth} < $depth;
 
-    if ($current && $params{on_release} && $current->{_depth} < $depth) {
+    if ($current && $params{on_release} && $depth_ok) {
         $current->{_on_release} ||= [];
         push @{$current->{_on_release}} => $params{on_release};
     }
@@ -237,7 +275,7 @@ sub context {
             _is_spawn   => [$pkg, $file, $line, $sub],
         },
         'Test2::API::Context'
-    ) if $current && $current->{_depth} < $depth;
+    ) if $current && $depth_ok;
 
     # Handle error condition of bad level
     if ($current) {
@@ -246,7 +284,7 @@ sub context {
                 unless $current->{_is_canon};
 
             _depth_error($current, [$pkg, $file, $line, $sub, $depth])
-                unless $current->{_depth} < $depth;
+                unless $depth_ok;
         }
 
         $current->release if $current->{_is_canon};
@@ -394,7 +432,10 @@ sub intercept(&) {
 }
 
 sub run_subtest {
-    my ($name, $code, $buffered, @args) = @_;
+    my ($name, $code, $params, @args) = @_;
+
+    $params = { buffered => $params } unless ref $params;
+    my $buffered = delete $params->{buffered};
 
     my $ctx = context();
 
@@ -405,12 +446,19 @@ sub run_subtest {
     my $stack = $ctx->stack || $STACK;
     my $hub = $stack->new_hub(
         class => 'Test2::Hub::Subtest',
+        %$params,
     );
 
     my @events;
     $hub->set_nested( $parent->isa('Test2::Hub::Subtest') ? $parent->nested + 1 : 1 );
     $hub->listen(sub { push @events => $_[1] });
-    $hub->format(undef) if $buffered;
+
+    if ($buffered) {
+        if (my $format = $hub->format) {
+            my $hide = $format->can('hide_buffered') ? $format->hide_buffered : 1;
+            $hub->format(undef) if $hide;
+        }
+    }
 
     my ($ok, $err, $finished);
     T2_SUBTEST_WRAPPER: {
@@ -448,10 +496,11 @@ sub run_subtest {
     my $pass = $ok && $hub->is_passing;
     my $e = $ctx->build_event(
         'Subtest',
-        pass => $pass,
-        name => $name,
-        buffered  => $buffered,
-        subevents => \@events,
+        pass       => $pass,
+        name       => $name,
+        subtest_id => $hub->id,
+        buffered   => $buffered,
+        subevents  => \@events,
     );
 
     my $plan_ok = $hub->check_plan;
@@ -481,10 +530,6 @@ __END__
 
 Test2::API - Primary interface for writing Test2 based testing tools.
 
-=head1 EXPERIMENTAL RELEASE
-
-This is an experimental release. Using this right now is not recommended.
-
 =head1 ***INTERNALS NOTE***
 
 B<The internals of this package are subject to change at any time!> The public
@@ -559,6 +604,8 @@ generated by the test system:
     use Test2::API qw{
         test2_init_done
         test2_stack
+        test2_set_is_end
+        test2_get_is_end
         test2_ipc
         test2_formatter_set
         test2_formatter
@@ -611,6 +658,12 @@ what you can and cannot do with a context once it is obtained.
 B<Note> This function will throw an exception if you ignore the context object
 it returns.
 
+B<Note> On perls 5.14+ a depth check is used to insure there are no context
+leaks. This cannot be safely done on older perls due to
+L<https://rt.perl.org/Public/Bug/Display.html?id=127774>
+You can forcefully enable it either by setting C<$ENV{T2_CHECK_DEPTH} = 1> or
+C<$Test2::API::DO_DEPTH_CHECK = 1> B<BEFORE> loading L<Test2::API>.
+
 =head3 OPTIONAL PARAMETERS
 
 All parameters to C<context> are optional.
@@ -772,7 +825,7 @@ Usage:
     }
 
 Using this inside your test tool takes care of a lot of boilerplate for you. It
-will ensure a context is aquired. It will capture and rethrow any exception. It
+will ensure a context is acquired. It will capture and rethrow any exception. It
 will insure the context is released when you are done. It preserves the
 subroutine call context (array, scalar, void).
 
@@ -809,9 +862,9 @@ Useage:
 =back
 
 This tool will hide a context for the provided block of code. This means any
-tools run inside the block will get a completely new context if they aquire
+tools run inside the block will get a completely new context if they acquire
 one. The new context will be inherited by tools nested below the one that
-aquired it.
+acquired it.
 
 This will normally hide the current context for the top hub. If you need to
 hide the context for a different hub you can pass in the optional C<$hid>
@@ -841,6 +894,10 @@ Usage:
 
     run_subtest($NAME, \&CODE, $BUFFERED, @ARGS)
 
+    # or
+
+    run_subtest($NAME, \&CODE, \%PARAMS, @ARGS)
+
 This will run the provided codeblock with the args in C<@args>. This codeblock
 will be run as a subtest. A subtest is an isolated test state that is condensed
 into a single L<Test2::Event::Subtest> event, which contains all events
@@ -858,12 +915,15 @@ The name of the subtest.
 
 The code to run inside the subtest.
 
-=item $BUFFERED
+=item $BUFFERED or \%PARAMS
 
-If this is true then the subtest will be buffered. In a buffered subtest the
-child events are hidden from the formatter, the formatter will only recieve the
-final L<Test2:Event::Subtest> event. In an unbuffered subtest the formatter
-will see all events as they happen, as well as the final one.
+If this is a simple scalar then it will be treated as a boolean for the
+'buffered' setting. If this is a hash reference then it wil be used as a
+parameters hash. The param hash will be used for hub construction (with the
+'buffered' key removed).
+
+If this is true, or a hashref with a true value for the 'buffered' key, then
+the subtest will be buffered.
 
 =item @ARGS
 
@@ -871,6 +931,52 @@ Any extra arguments you want passed into the subtest code.
 
 =back
 
+=head3 BUFFERED VS UNBUFFERED (OR STREAMED)
+
+Normally all events inside and outside a subtest are sent to the formatter
+immedietly by the hub. Sometimes it is desirable to hold off sending events
+within a subtest until the subtest is complete. This usually depends on the
+formatter being used.
+
+=over 4
+
+=item Things not effected by this flag
+
+In both cases events are generated and stored in an array. This array is
+eventually used to populate the C<subevents> attribute on the
+L<Test2::Event::Subtest> event that is generated at the end of the subtest.
+This flag has no effect on this part, it always happens.
+
+At the end of the subtest the final L<Test2::Event::Subtest> event is sent to
+the formatter.
+
+=item Things that are effected by this flag
+
+The C<buffered> attribute of the L<Test2::Event::Subtest> event will be set to
+the value of this flag. This means any formatter, listener, etc which looks at
+the event will know if it was buffered.
+
+=item Things that are formatter dependant
+
+Events within a buffered subtest may or may not be sent to the formatter as
+they happen. If a formatter fails to specify then the default is to B<NOT SEND>
+the events as they are generated, instead the formatter can pull them from the
+C<subevents> attribute.
+
+A formatter can specify by implementing the C<hide_buffered()> method. If this
+method returns true then events generated inside a buffered subtest will not be
+sent independantly of the final subtest event.
+
+=back
+
+An example of how this is used is the L<Test2::Formatter::TAP> formatter. For
+unbuffered subtests the events are rendered as they are generated. At the end
+of the subtest the final subtest event is rendered, but the C<subevents>
+attribute is ignored. For buffered subtests the opposite occurs, the events are
+NOT rendered as they are generated, instead the C<subevents> attribute is used
+to render them all at once. This is useful when running subtests tests in
+parallel, without it the subtests would be garbled.
+
 =head1 OTHER API EXPORTS
 
 Exports in this section are not commonly needed. These all have the 'test2_'
@@ -902,6 +1008,22 @@ This will simply return the boolean value of the loaded flag. If Test2 has
 finished loading this will be true, otherwise false. Loading is considered
 complete the first time a tool requests a context.
 
+=item test2_set_is_end()
+
+=item test2_set_is_end($bool)
+
+This is used to toggle Test2's belief that the END phase has already started.
+With no arguments this will set it to true. With arguments it will set it to
+the first argument's value.
+
+This is used to prevent the use of C<caller()> in END blocks which can cause
+segfaults. This is only necessary in some persistent environments that may have
+multiple END phases.
+
+=item $bool = test2_get_is_end()
+
+Check if test2 believes it is the END phase.
+
 =item $stack = test2_stack()
 
 This will return the global L<Test2::API::Stack> instance. If this has not
@@ -950,37 +1072,37 @@ Add a callback that will be called when Test2 is finished loading. This
 means the callback will be run once, the first time a context is obtained.
 If Test2 has already finished loading then the callback will be run immedietly.
 
-=item test2_add_callback_context_aquire(sub { ... })
+=item test2_add_callback_context_acquire(sub { ... })
 
-Add a callback that will be called every time someone tries to aquire a
+Add a callback that will be called every time someone tries to acquire a
 context. This will be called on EVERY call to C<context()>. It gets a single
 argument, a reference the the hash of parameters being used the construct the
 context. This is your chance to change the parameters by directly altering the
 hash.
 
-    test2_add_callback_context_aquire(sub {
+    test2_add_callback_context_acquire(sub {
         my $params = shift;
         $params->{level}++;
     });
 
 This is a very scary API function. Please do not use this unless you need to.
-This is here for L<Test::Builder> and backwards compatability. This has you
+This is here for L<Test::Builder> and backwards compatibility. This has you
 directly manipulate the hash instead of returning a new one for performance
 reasons.
 
 =item test2_add_callback_context_init(sub { ... })
 
 Add a callback that will be called every time a new context is created. The
-callback will recieve the newly created context as its only argument.
+callback will receive the newly created context as its only argument.
 
 =item test2_add_callback_context_release(sub { ... })
 
 Add a callback that will be called every time a context is released. The
-callback will recieve the released context as its only argument.
+callback will receive the released context as its only argument.
 
-=item @list = test2_list_context_aquire_callbacks()
+=item @list = test2_list_context_acquire_callbacks()
 
-Return all the context aquire callback references.
+Return all the context acquire callback references.
 
 =item @list = test2_list_context_init_callbacks()
 
@@ -1122,7 +1244,7 @@ can be added to this package.
 =head1 SOURCE
 
 The source code repository for Test2 can be found at
-F<http://github.com/Test-More/Test2/>.
+F<http://github.com/Test-More/test-more/>.
 
 =head1 MAINTAINERS
 
@@ -1142,7 +1264,7 @@ F<http://github.com/Test-More/Test2/>.
 
 =head1 COPYRIGHT
 
-Copyright 2015 Chad Granum E<lt>exodist7@gmail.comE<gt>.
+Copyright 2016 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
