@@ -2,8 +2,10 @@ package Test2::Event::Subtest;
 use strict;
 use warnings;
 
+our $VERSION = '0.000044';
+
 use base 'Test2::Event::Ok';
-use Test2::Util::HashBase qw{subevents buffered};
+use Test2::Util::HashBase qw{subevents buffered subtest_id};
 
 sub init {
     my $self = shift;
@@ -11,6 +13,22 @@ sub init {
     $self->{+SUBEVENTS} ||= [];
 }
 
+sub summary {
+    my $self = shift;
+
+    my $name = $self->{+NAME} || "Nameless Subtest";
+
+    my $todo = $self->{+TODO};
+    if ($todo) {
+        $name .= " (TODO: $todo)";
+    }
+    elsif (defined $todo) {
+        $name .= " (TODO)"
+    }
+
+    return $name;
+}
+
 1;
 
 __END__
@@ -23,10 +41,6 @@ __END__
 
 Test2::Event::Subtest - Event for subtest types
 
-=head1 EXPERIMENTAL RELEASE
-
-This is an experimental release. Using this right now is not recommended.
-
 =head1 DESCRIPTION
 
 This class represents a subtest. This class is a subclass of
@@ -52,7 +66,7 @@ is false it means all subevents render as they are produced.
 =head1 SOURCE
 
 The source code repository for Test2 can be found at
-F<http://github.com/Test-More/Test2/>.
+F<http://github.com/Test-More/test-more/>.
 
 =head1 MAINTAINERS
 
@@ -72,7 +86,7 @@ F<http://github.com/Test-More/Test2/>.
 
 =head1 COPYRIGHT
 
-Copyright 2015 Chad Granum E<lt>exodist7@gmail.comE<gt>.
+Copyright 2016 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
