@@ -2,6 +2,8 @@ package Test2::Hub::Interceptor::Terminator;
 use strict;
 use warnings;
 
+our $VERSION = '0.000044';
+
 1;
 
 __END__
@@ -15,14 +17,10 @@ __END__
 Test2::Hub::Interceptor::Terminator - Exception class used by
 Test2::Hub::Interceptor
 
-=head1 EXPERIMENTAL RELEASE
-
-This is an experimental release. Using this right now is not recommended.
-
 =head1 SOURCE
 
 The source code repository for Test2 can be found at
-F<http://github.com/Test-More/Test2/>.
+F<http://github.com/Test-More/test-more/>.
 
 =head1 MAINTAINERS
 
@@ -42,7 +40,7 @@ F<http://github.com/Test-More/Test2/>.
 
 =head1 COPYRIGHT
 
-Copyright 2015 Chad Granum E<lt>exodist7@gmail.comE<gt>.
+Copyright 2016 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
